SET SERVEROUTPUT ON SIZE 1000000

DECLARE
  
  v_major INTEGER := 0;
  v_minor INTEGER := 0;
  v_patch INTEGER := 0;
  v_upgrade_state VARCHAR2(24);
  
  v_req_major     CONSTANT INTEGER  := 3;
  v_req_minor     CONSTANT INTEGER  := 3;
  v_req_patch_min CONSTANT INTEGER  := 0;
  v_req_patch_max CONSTANT INTEGER  := 0;
  
  v_next_major CONSTANT INTEGER := 3;
  v_next_minor CONSTANT INTEGER := 4;
  v_next_patch CONSTANT INTEGER := 0;
  
  -- verify if the 'Online Index Build' option is supported (disabled in Oracle XE)
  online_index_build V$OPTION.VALUE%TYPE := NULL;
 

--
-- Add a column to an existing table
PROCEDURE add_column(table_name  VARCHAR2
                    ,column_name VARCHAR2
                    ,column_stmt VARCHAR2)
IS
  column_exists EXCEPTION;                  --column already exists
  PRAGMA EXCEPTION_INIT(column_exists,-1430); -- associate with error ORA-01430
BEGIN
  EXECUTE IMMEDIATE ('ALTER TABLE ' || table_name || ' ADD ' || column_name || ' ' || column_stmt);
  DBMS_OUTPUT.PUT_LINE('INFO: Column [' || table_name || '.' || column_name || '] added');
EXCEPTION
WHEN column_exists THEN
  DBMS_OUTPUT.PUT_LINE('INFO: Column [' || table_name || '.' || column_name || '] already exists. Skip it');
  RETURN;
END add_column;

--
-- Procedure to check if a table already exists
-- If it does not exist it creates a table with a column
-- the other columns will be added with the add in the main 
PROCEDURE tableExistance(tableName VARCHAR2, columnName VARCHAR ) 
IS
   counter NUMBER;
BEGIN
    SELECT COUNT (*) INTO counter FROM user_tables
    WHERE  table_name = tableName;

    IF counter = 0 THEN
        DBMS_OUTPUT.PUT_LINE('INFO: Creating Create table ' || tableName || ' (' || columnName || ')');
        EXECUTE IMMEDIATE ('CREATE TABLE ' || tableName || ' (' || columnName || ')');
    END IF;
END tableExistance;

--
-- Drop Index procedure
PROCEDURE drop_index_if_exists(index_name VARCHAR2)
IS
  index_inexistent EXCEPTION;                   -- index does not exists
  PRAGMA EXCEPTION_INIT(index_inexistent,-1418); -- associate with error ORA-01418
BEGIN
  EXECUTE IMMEDIATE ('DROP INDEX ' || index_name );
EXCEPTION
WHEN index_inexistent THEN
  RETURN;
END drop_index_if_exists;

--
-- Drop constraint procedure
PROCEDURE drop_constraint_if_exists(
  table_name VARCHAR2,
  constraint_name VARCHAR2)
IS
  constraint_inexistent EXCEPTION;                   -- constraint does not exists
  PRAGMA EXCEPTION_INIT(constraint_inexistent,-2443); -- associate with error ORA-02443
BEGIN
  EXECUTE IMMEDIATE('alter table ' || table_name || ' drop CONSTRAINT ' || constraint_name);
EXCEPTION
WHEN constraint_inexistent THEN
  RETURN;
END drop_constraint_if_exists;

--
-- Main
BEGIN

  DBMS_OUTPUT.PUT_LINE('Applying Patch...');
  -- Get schema version
  SELECT MAJOR,MINOR,PATCH INTO v_major,v_minor,v_patch FROM T_SCHEMA_VERS WHERE ROWNUM=1;
  -- check version
  IF v_major = v_req_major AND  v_minor = v_req_minor AND (v_patch BETWEEN  v_req_patch_min AND v_req_patch_max) THEN
      -- apply patch
      
      -- Add a schema state column that can be used to check if a previous upgrade attempt failed
      DBMS_OUTPUT.PUT_LINE('Upgrading T_SCHEMA_VERS...');
      -- set schema state 
      EXECUTE IMMEDIATE('UPDATE T_SCHEMA_VERS SET STATE=''Upgrading''');
      DBMS_OUTPUT.PUT_LINE('Upgrading schema ' || v_major || '.' || v_minor || '.' || v_patch || ' to ' || v_next_major || '.' || v_next_minor || '.' || v_next_patch || ' ...');

      --
      -- Add the foreign keys added

      drop_constraint_if_exists('t_channel_vo_share','channel_vo_share_fk');
      EXECUTE IMMEDIATE ('ALTER TABLE t_channel_vo_share ADD CONSTRAINT channel_vo_share_fk FOREIGN KEY (channel_name) REFERENCES t_channel (channel_name)');

      drop_constraint_if_exists('t_channel_acl','channel_acl_fk');
      EXECUTE IMMEDIATE ('ALTER TABLE t_channel_acl ADD CONSTRAINT channel_acl_fk FOREIGN KEY (channel_name) REFERENCES t_channel (channel_name)');

      -- Remove the unnecessary indexes
      drop_index_if_exists( 'channel_vo_share_channel_name'); 
      drop_index_if_exists( 'channel_acl_channel_name' );
      drop_index_if_exists( 'vo_acl_vo_name' );
      drop_index_if_exists( 'site_group_gname_id');
      drop_index_if_exists( 'file_file_state');
      drop_index_if_exists( 'transfer_request_id' );
      drop_index_if_exists( 'stagereq_unique_idx' );

      -- -----------------------------------------------------------------------------------------------
      -- History tables
      -- -----------------------------------------------------------------------------------------------

      --check if the table exists and create it if not
      tableExistance('T_JOB_HISTORY', 'job_id CHAR(36) CONSTRAINT job_h_job_id_pk PRIMARY KEY');
      -- add t_job_history properties
      DBMS_OUTPUT.PUT_LINE('Adding T_JOB_HISTORY properties...');
      add_column('t_job_history','job_id','CHAR(36) CONSTRAINT job_h_job_id_pk PRIMARY KEY');
      add_column('t_job_history','job_state','VARCHAR2(32)');
      add_column('t_job_history','cancel_job','CHAR(1)');
      add_column('t_job_history','job_params','VARCHAR2(255)');
      add_column('t_job_history','source','VARCHAR2(255)');
      add_column('t_job_history','dest','VARCHAR2(255)');
      add_column('t_job_history','source_se','VARCHAR2(255)');
      add_column('t_job_history','dest_se','VARCHAR2(255)');
      add_column('t_job_history','user_dn','VARCHAR2(1024)');
      add_column('t_job_history','agent_dn','VARCHAR2(1024)');
      add_column('t_job_history','user_cred','VARCHAR2(255)');
      add_column('t_job_history','cred_id','VARCHAR2(100)');
      add_column('t_job_history','voms_cred','BLOB');
      add_column('t_job_history','vo_name','VARCHAR2(50)');
      add_column('t_job_history','channel_name','VARCHAR2(32)');
      add_column('t_job_history','reason','VARCHAR2(1024)');
      add_column('t_job_history','submit_time','TIMESTAMP WITH TIME ZONE');
      add_column('t_job_history','finish_time','TIMESTAMP WITH TIME ZONE');
      add_column('t_job_history','priority','INTEGER DEFAULT 3');
      add_column('t_job_history','submit_host','VARCHAR2(255)');
      add_column('t_job_history','max_time_in_queue','INTEGER');
      add_column('t_job_history','space_token','VARCHAR2(255)');
      add_column('t_job_history','storage_class','VARCHAR2(255)');
      add_column('t_job_history','myproxy_server','VARCHAR2(255)');
      add_column('t_job_history','src_catalog','VARCHAR2(1024)');
      add_column('t_job_history','src_catalog_type','VARCHAR2(1024)');
      add_column('t_job_history','dest_catalog','VARCHAR2(1024)');
      add_column('t_job_history','dest_catalog_type','VARCHAR2(1024)');
      add_column('t_job_history','internal_job_params','VARCHAR2(255)');
      add_column('t_job_history','overwrite_flag','CHAR(1)');
      add_column('t_job_history','job_finished','TIMESTAMP');
      add_column('t_job_history','source_space_token','VARCHAR2(255)');
      add_column('t_job_history','source_token_description','VARCHAR2(255)');
      add_column('t_job_history','copy_pin_lifetime','INTEGER default NULL');
      add_column('t_job_history','lan_connection','CHAR(1) default NULL');
      add_column('t_job_history','fail_nearline','CHAR(1) default NULL');
            
      
      --check if the table exists if not create it adding just one column
      -- The rest will be added with the add_column procedure
      tableExistance('T_FILE_HISTORY','file_id	INTEGER CONSTRAINT file_h_file_id_pk PRIMARY KEY');
          
      -- add t_file_history properties
      DBMS_OUTPUT.PUT_LINE('Adding T_FILE_HISTORY properties...');
      add_column('t_file_history','file_id','INTEGER CONSTRAINT file_h_file_id_pk PRIMARY KEY');
      add_column('t_file_history','job_id','CHAR(36)');
      add_column('t_file_history','file_state','VARCHAR2(32)');
      add_column('t_file_history','logical_name','VARCHAR2(1100)');
      add_column('t_file_history','source_surl','VARCHAR2(1100)');
      add_column('t_file_history','dest_surl','VARCHAR2(1100)');
      add_column('t_file_history','agent_dn','VARCHAR2(1024)');
      add_column('t_file_history','error_scope','VARCHAR2(32)');
      add_column('t_file_history','error_phase','VARCHAR2(32)');
      add_column('t_file_history','reason_class','VARCHAR2(32)');
      add_column('t_file_history','reason','VARCHAR2(1024)');
      add_column('t_file_history','num_failures','INTEGER');
      add_column('t_file_history','current_failures','INTEGER');
      add_column('t_file_history','catalog_failures','INTEGER');
      add_column('t_file_history','prestage_failures','INTEGER');
      add_column('t_file_history','filesize','INTEGER');
      add_column('t_file_history','checksum','VARCHAR2(100)');
      add_column('t_file_history','finish_time','TIMESTAMP WITH TIME ZONE');
      add_column('t_file_history','internal_file_params','VARCHAR2(255)');
      add_column('t_file_history','job_finished','TIMESTAMP');
   
      
      -- chek the existance of t_transfer_history
      tableExistance('T_TRANSFER_HISTORY', 'request_id VARCHAR2(255)');
      
      -- add t_transfer_history properties
      DBMS_OUTPUT.PUT_LINE('Adding T_TRANSFER_HISTORY properties...');
      add_column('t_transfer_history','request_id','VARCHAR2(255)');
      add_column('t_transfer_history','file_id','INTEGER');
      add_column('t_transfer_history','transfer_id','INTEGER');
      add_column('t_transfer_history','job_id','CHAR(36)');
      add_column('t_transfer_history','transfer_state','VARCHAR2(32)');
      add_column('t_transfer_history','source_turl','VARCHAR2(1100)');
      add_column('t_transfer_history','dest_turl','VARCHAR2(1100)');
      add_column('t_transfer_history','source_srm_token','VARCHAR2(256)');
      add_column('t_transfer_history','dest_srm_token','VARCHAR2(256)');
      add_column('t_transfer_history','source_host','VARCHAR2(256)');
      add_column('t_transfer_history','dest_host','VARCHAR2(256)');
      add_column('t_transfer_history','transfer_time','TIMESTAMP WITH TIME ZONE');
      add_column('t_transfer_history','prepare_time','TIMESTAMP WITH TIME ZONE');
      add_column('t_transfer_history','duration','NUMBER(12,2)');
      add_column('t_transfer_history','src_prep_duration','NUMBER(12,3)');
      add_column('t_transfer_history','dest_prep_duration','NUMBER(12,3)');
      add_column('t_transfer_history','tx_duration','NUMBER(12,3)');
      add_column('t_transfer_history','src_final_duration','NUMBER(12,3)');
      add_column('t_transfer_history','dest_final_duration','NUMBER(12,3)');
      add_column('t_transfer_history','bytes_written','INTEGER');
      add_column('t_transfer_history','throughput','NUMBER');
      add_column('t_transfer_history','error_scope','VARCHAR2(32)');
      add_column('t_transfer_history','error_phase','VARCHAR2(32)');
      add_column('t_transfer_history','reason_class','VARCHAR2(32)');
      add_column('t_transfer_history','reason','VARCHAR2(1024)');
      add_column('t_transfer_history','filesize','INTEGER');
      add_column('t_transfer_history','transfer_type','VARCHAR(32)');
      add_column('t_transfer_history','finish_time','TIMESTAMP WITH TIME ZONE');
      add_column('t_transfer_history','job_finished','TIMESTAMP DEFAULT NULL');  

      drop_constraint_if_exists('t_transfer_history','transfer_h_pk');
      EXECUTE IMMEDIATE ('ALTER TABLE t_transfer_history ADD CONSTRAINT transfer_h_pk PRIMARY KEY (request_id, file_id)');
           
      -- t_history_purge_log
      tableExistance('T_HISTORY_PURGE_LOG','starttime   TIMESTAMP(6) WITH TIME ZONE');
      add_column('t_history_purge_log','finishtime','TIMESTAMP(6) WITH TIME ZONE');
      add_column('t_history_purge_log','jobs','NUMBER');
      add_column('t_history_purge_log','files','NUMBER');
      add_column('t_history_purge_log','transfers','NUMBER');
      add_column('t_history_purge_log','errcode','NUMBER');
      add_column('t_history_purge_log','errmsg','VARCHAR2(2048)');

      -- temporary table for the jobs to be deleted
      tableExistance('X_PURGE_JOBIDS', 'job_id CHAR(36) CONSTRAINT x_purge_jobids_pk PRIMARY KEY');
      
      -- temporary table for the files to be deleted
      tableExistance('X_PURGE_FILEIDS','file_id  NUMBER(38) CONSTRAINT x_purge_fileids_pk PRIMARY KEY');

      -- temporary table for the jobs to be deleted
      tableExistance('X_JOBIDS', 'job_id CHAR(36) CONSTRAINT x_jobids_pk PRIMARY KEY');
      
      -- temporary table for the files to be deleted
      tableExistance('X_FILEIDS','file_id  NUMBER(38) CONSTRAINT x_fileids_pk PRIMARY KEY');
            
      tableExistance('T_HISTORY_LOG','starttime   TIMESTAMP(6) WITH TIME ZONE');
      add_column('t_history_log','finishtime','TIMESTAMP(6) WITH TIME ZONE');
      add_column('t_history_log','jobs','NUMBER');
      add_column('t_history_log','files','NUMBER');
      add_column('t_history_log','transfers','NUMBER');
      add_column('t_history_log','errcode','NUMBER');
      add_column('t_history_log','errmsg','VARCHAR2(2048)');
     
      -- tables  for the plug-ins
      tableExistance('T_FTS_PLUGIN','packagename VARCHAR2(100) NOT NULL');
      add_column('t_fts_plugin','description','VARCHAR2(1024) NOT NULL');
      add_column('t_fts_plugin','author','VARCHAR2(100) NOT NULL');
      add_column('t_fts_plugin','dateadded','TIMESTAMP(6) WITH TIME ZONE DEFAULT systimestamp');
      add_column('t_fts_plugin','packageversion','VARCHAR2(10) NOT NULL');
      add_column('t_fts_plugin','requirepackageschemamin','VARCHAR2(10)');
      add_column('t_fts_plugin','requirepackageschemamax','VARCHAR2(10)');
      add_column('t_fts_plugin','requirecoreschemamin','VARCHAR2(10) NOT NULL');
      add_column('t_fts_plugin','requirecoreschemamax','VARCHAR2(10)');
      add_column('t_fts_plugin','requiremaxdbstate','VARCHAR2(15) NOT NULL CONSTRAINT fts_plugin_reqdbstate CHECK (requireMaxDbState IN (''PRODUCTION'', ''TEST''))');
      
      tableExistance('T_FTS_PLUGIN_SCHEMA','packagename VARCHAR2(100) NOT NULL CONSTRAINT fts_plugin_schema_pk PRIMARY KEY');
      add_column('t_fts_plugin_schema','schemaversion','VARCHAR2(10)');
            
      DBMS_OUTPUT.PUT_LINE('Updating schema version...');
      UPDATE T_SCHEMA_VERS SET MAJOR=v_next_major, MINOR=v_next_minor, PATCH=v_next_patch; 
      EXECUTE IMMEDIATE ('UPDATE T_SCHEMA_VERS SET STATE=NULL'); 
      DBMS_OUTPUT.PUT_LINE('SUCCESS: Patch applied. New version is ' || TO_CHAR(v_next_major) || '.' || TO_CHAR(v_next_minor) || '.' || TO_CHAR(v_next_patch) );
  ELSIF v_major = v_next_major AND  v_minor = v_next_minor AND v_patch = v_next_patch THEN
     DBMS_OUTPUT.PUT_LINE('SUCCESS: Patch already applied. Version is ' || TO_CHAR(v_next_major) || '.' || TO_CHAR(v_next_minor) || '.' || TO_CHAR(v_next_patch) );
  ELSE
     DBMS_OUTPUT.PUT_LINE('ERROR: Cannot apply patch. Version ' || TO_CHAR(v_major) || '.' || TO_CHAR(v_minor) || '.' || TO_CHAR(v_patch) || ' does not match the required one (' || TO_CHAR(v_req_major) || '.' || TO_CHAR(v_req_minor) || '.' || TO_CHAR(v_req_patch_min) || '-' ||  TO_CHAR(v_req_patch_max) || ')');
  END IF;
END;
/
exit;
