SET SERVEROUTPUT ON SIZE 1000000

DECLARE

  v_major INTEGER := 0;
  v_minor INTEGER := 0;
  v_patch INTEGER := 0;
  v_upgrade_state VARCHAR2(24);

  v_req_major     CONSTANT INTEGER  := 3;
  v_req_minor     CONSTANT INTEGER  := 4;
  v_req_patch_min CONSTANT INTEGER  := 0;
  v_req_patch_max CONSTANT INTEGER  := 1;

  v_next_major CONSTANT INTEGER := 3;
  v_next_minor CONSTANT INTEGER := 5;
  v_next_patch CONSTANT INTEGER := 0;

  -- verify if the 'Online Index Build' option is supported (disabled in Oracle XE)
  online_index_build V$OPTION.VALUE%TYPE := NULL;


--
-- Add a column to an existing table
PROCEDURE add_column(table_name  VARCHAR2
                    ,column_name VARCHAR2
                    ,column_stmt VARCHAR2)
IS
  column_exists EXCEPTION;                  --column already exists
  PRAGMA EXCEPTION_INIT(column_exists,-1430); -- associate with error ORA-01430
BEGIN
  EXECUTE IMMEDIATE ('ALTER TABLE ' || table_name || ' ADD ' || column_name || ' ' || column_stmt);
  DBMS_OUTPUT.PUT_LINE('INFO: Column [' || table_name || '.' || column_name || '] added');
EXCEPTION
WHEN column_exists THEN
  DBMS_OUTPUT.PUT_LINE('INFO: Column [' || table_name || '.' || column_name || '] already exists. Skip it');
  RETURN;
END add_column;

--
-- Procedure to check if a table already exists
-- If it does not exist it creates a table with a column
-- the other columns will be added with the add in the main 
PROCEDURE tableExistance(tableName VARCHAR2, columnName VARCHAR ) 
IS
   counter NUMBER;
BEGIN
    SELECT COUNT (*) INTO counter FROM user_tables
    WHERE  table_name = tableName;

    IF counter = 0 THEN
        DBMS_OUTPUT.PUT_LINE('INFO: Creating Create table ' || tableName || ' (' || columnName || ')');
        EXECUTE IMMEDIATE ('CREATE TABLE ' || tableName || ' (' || columnName || ')');
    END IF;
END tableExistance;



-- Main
BEGIN

  DBMS_OUTPUT.PUT_LINE('Applying Patch...');
  -- Get schema version
  SELECT MAJOR,MINOR,PATCH INTO v_major,v_minor,v_patch FROM T_SCHEMA_VERS WHERE ROWNUM=1;
  -- check version
  IF v_major = v_req_major AND  v_minor = v_req_minor AND (v_patch BETWEEN  v_req_patch_min AND v_req_patch_max) THEN
      -- apply patch

      -- Add a schema state column that can be used to check if a previous upgrade attempt failed
      DBMS_OUTPUT.PUT_LINE('Upgrading T_SCHEMA_VERS...');
      -- set schema state
      EXECUTE IMMEDIATE('UPDATE T_SCHEMA_VERS SET STATE=''Upgrading''');
      DBMS_OUTPUT.PUT_LINE('Upgrading schema ' || v_major || '.' || v_minor || '.' || v_patch || ' to ' || v_next_major || '.' || v_next_minor || '.' || v_next_patch || ' ...');

     
      -- Add the table T_SE
      --check if the table exists and create it if not

      tableExistance('T_SE', 'ENDPOINT VARCHAR2(1024)');
      add_column('T_SE','se_type','varchar(30)');
      add_column('T_SE','site','varchar2(100) NOT NULL');
      add_column('T_SE','name','varchar2(255) CONSTRAINT se_name_id_pk PRIMARY KEY');
      add_column('T_SE','STATE','varchar2(30)');
      add_column('T_SE','VERSION','varchar2(30)');
      add_column('T_SE','HOST','varchar2(100)');

      tableExistance('T_SE_ACL','NAME varchar2(255)');
      add_column('T_SE_ACL','vo','varchar2(32)'); 
      -- Add the primary key 
      EXECUTE IMMEDIATE ('ALTER TABLE T_SE_ACL ADD CONSTRAINT se_acl_pk PRIMARY KEY (NAME,VO)');


      DBMS_OUTPUT.PUT_LINE('Updating schema version...');
      UPDATE T_SCHEMA_VERS SET MAJOR=v_next_major, MINOR=v_next_minor, PATCH=v_next_patch; 
      EXECUTE IMMEDIATE ('UPDATE T_SCHEMA_VERS SET STATE=NULL'); 
      DBMS_OUTPUT.PUT_LINE('SUCCESS: Patch applied. New version is ' || TO_CHAR(v_next_major) || '.' || TO_CHAR(v_next_minor) || '.' || TO_CHAR(v_next_patch) );
 ELSIF v_major = v_next_major AND  v_minor = v_next_minor AND v_patch = v_next_patch THEN
     DBMS_OUTPUT.PUT_LINE('SUCCESS: Patch already applied. Version is ' || TO_CHAR(v_next_major) || '.' || TO_CHAR(v_next_minor) || '.' || TO_CHAR(v_next_patch) );
  ELSE
     DBMS_OUTPUT.PUT_LINE('ERROR: Cannot apply patch. Version ' || TO_CHAR(v_major) || '.' || TO_CHAR(v_minor) || '.' || TO_CHAR(v_patch) || ' does not match the required one (' || TO_CHAR(v_req_major) || '.' || TO_CHAR(v_req_minor) || '.' || TO_CHAR(v_req_patch_min) || '-' ||  TO_CHAR(v_req_patch_max) || ')');
  END IF;
END;
/
exit;
